rm(list = ls())
## Implementation of CHAID algorithm for student academic performance data
# Data source: https://archive.ics.uci.edu/ml/datasets/Student+Performance


#------- Data upload & manipulation

#setwd("C:/Users/local_sahooi/Desktop/Research/SUMMER 2021 REU Files/GPA/student/")
dataset  <- read.table("student-mat.csv", sep = ";", header = T)

# Response variable
res_bin  <- as.factor(cut(dataset$G3, breaks = c(-Inf, 15, Inf), 
                          labels = c("Low score", "High score")))


# Predictor Variables
G1_bin   <- as.factor(cut(dataset$G1, breaks = c(-Inf, 15, Inf), 
                          labels = c("Low score", "High score")))
G2_bin   <- as.factor(cut(dataset$G2, breaks = c(-Inf, 15, Inf), 
                          labels = c("Low score", "High score")))
pred_var <- cbind(dataset[, 1:30], G1_bin, G2_bin)

data_merge <- cbind(pred_var, res_bin)
data_merge[] <- lapply(data_merge, factor)
data_merge <- data_merge[1:390, ]


#------- Installing required packages

#install and load necessary packages/libraries
#install.packages('splitstackshape')
#install.packages("CHAID", repos="http://R-Forge.R-project.org")
#install.packages('caret')
library(splitstackshape) 
library(CHAID)
library(caret)

#------------------

# CHAID algorithm & plotting a CHAID tree

sampling <- stratified(data_merge, c("res_bin"), .7, bothSets = TRUE)
training <- sampling$SAMP1
testing  <- sampling$SAMP2

chaid.model <- chaid(res_bin ~  ., data = training, 
                     control = chaid_control(alpha2 = 0.1, alpha4 = 0.1,
                                             minsplit = 10, minbucket = 2))

pmodel <- predict(chaid.model)
cm <- confusionMatrix(pmodel, training$res_bin)
cm
chaid.model

plot(chaid.model, main = "An example of a CHAID tree", type = 'simple')



#------- Calculating training and testing accuracy
# based on 500 iterations

test.accuracy <- c()
train.accuracy <- c()

for (i in 1:500){
  sampling <- stratified(data_merge, c("res_bin"), .7, bothSets = TRUE)
  training <- sampling$SAMP1
  testing  <- sampling$SAMP2
  
  chaid.forest1 <- chaid(res_bin ~  ., data = training, 
                         control = chaid_control(alpha2 = 0.1, alpha4 = 0.1, 
                                                 minsplit = 10, minbucket = 2))
  
  pmodel.forest1 <- predict(chaid.forest1)
  cm <- confusionMatrix(pmodel.forest1, training$res_bin)
  train.accuracy <- append(train.accuracy, as.numeric(cm$overall[1]))
  
  cm.forest1 <- table(testing$res_bin, predict(chaid.forest1, newdata = testing))
  test.accuracy <- append(test.accuracy, mean(testing$res_bin == 
                                                predict(chaid.forest1, 
                                                        newdata = testing)))
  
  print(i)
}

avg.train.f1 <- mean(train.accuracy)
avg.train.f1

avg.test.f1 <- mean(test.accuracy)
avg.test.f1
