rm(list = ls())
## Implementation of `cforest' algorithm for student academic performance data
# Data source: https://archive.ics.uci.edu/ml/datasets/Student+Performance

#------- Data upload & manipulation

#setwd("C:/Users/local_sahooi/Desktop/Research/SUMMER 2021 REU Files/GPA/student/")
dataset  <- read.table("student-mat.csv", sep = ";", header = T)

# Response variable
res_bin  <- as.factor(cut(dataset$G3, breaks = c(-Inf, 15, Inf), 
                          labels = c("Low score", "High score")))


# Predictor Variables
G1_bin   <- as.factor(cut(dataset$G1, breaks = c(-Inf, 15, Inf), 
                          labels = c("Low score", "High score")))
G2_bin   <- as.factor(cut(dataset$G2, breaks = c(-Inf, 15, Inf), 
                          labels = c("Low score", "High score")))
pred_var <- cbind(dataset[, 1:30], G1_bin, G2_bin)

data_merge <- cbind(pred_var, res_bin)
data_merge[] <- lapply(data_merge, factor)
data_merge <- data_merge[1:390, ]


#------- Installing required packages & Relevant Functions

#install and load necessary packages/libraries
#install.packages('splitstackshape')
#install.packages('party')
library(splitstackshape)
library(party)


#------- Relevant functions

# this function is used in the 500 iterations loop to get training and testing
# accuracies for our fitted `cforest' model. 

theStats <- function(training, testing){
  training.acc <- c()
  testing.acc <- c()
  dim <- dim(training)[1]
  for(i in 1:dim){ # training accuracy
    x <- training[i, i] / sum(training[i, ])
    training.acc <- append(training.acc, x)
  }
  for(i in 1:dim){ # testing accuracy
    x <- testing[i, i] / sum(testing[i, ])
    testing.acc <- append(testing.acc, x)
  }
  train <- mean(training.acc)
  test <- mean(testing.acc)
  df <- data.frame(train, test)
  return(df)
}


#------- Calculating training and testing Accuracy 
# based on 500 iterations

data_merge$res_bin <- factor(data_merge$res_bin, 
                        levels = c("Low score", "High score"), ordered = TRUE)
acc.tot <- data.frame(matrix(ncol = 2, nrow = 0))
names(acc.tot) <- c("Training Accuracy", "Testing Accuracy")

for(i in 1:500){
  sampling <- stratified(data_merge, c("res_bin"), .7, bothSets = TRUE)
  training <- sampling$SAMP1
  testing  <- sampling$SAMP2
  my.forest <- cforest(res_bin ~ ., data = training, 
                       control = cforest_unbiased(mtry = 6, minsplit = 10, 
                                                  minbucket = 2, ntree = 501))
  
  training.table <- table(predict(my.forest), training$res_bin)
  testing.table <- table(predict(my.forest, newdata = testing), testing$res_bin)
  avgstats <- theStats(training.table, testing.table)
  acc.tot = rbind(acc.tot, avgstats)
  
  print(i)
}

mean(acc.tot[, 1]) #mean training accuracy
mean(acc.tot[, 2]) #mean testing  accuracy


#------- Variable Importance 

# Computing (conditional) variable importance 

sampling <- stratified(data_merge, c("res_bin"), .7, bothSets = TRUE)
training <- sampling$SAMP1
testing  <- sampling$SAMP2
my.forest<- cforest(res_bin ~ ., data = training, 
                       control = cforest_unbiased(mtry = 6, minsplit = 10, 
                                                  minbucket = 2, ntree = 501))

# takes a while to run...
vimp.tot <- as.vector(varimp(my.forest, conditional = TRUE))
vimp.tot




